@@ -60,32 +60,38 @@ setMethod("ADC.fast", signature(dwi="array"),
           .dcemriWrapper("ADC.fast", dwi, bvalues, dwi.mask, verbose))
 
 .ADC.fast <- function(dwi, bvalues, dwi.mask, verbose=FALSE) {
-  if (length(dim(dwi)) != 4)  # Check dwi is a 4D array
+  if (length(dim(dwi)) != 4) { # Check dwi is a 4D array
     stop("Diffusion-weighted data must be a 4D array.")
-  if (!is.logical(dwi.mask))  # Check dyn.mask is logical
+  }
+  if (!is.logical(dwi.mask)) { # Check dyn.mask is logical
     stop("Mask must be logical.")
+  }
 
   nvalues <- length(bvalues)
   nvoxels <- sum(dwi.mask)
   
-  if (verbose)
+  if (verbose) {
     cat("  Deconstructing data...", fill=TRUE)
+  }
   dwi.mat <- matrix(dwi[dwi.mask], nvoxels)
   S0 <- D <- numeric(nvoxels)
 
-  if (verbose)
+  if (verbose) {
     cat("  Calculating S0 and D...", fill=TRUE)
-  for(i in 1:nvoxels) {
-    fit <- adc.lm(dwi.mat[i,], bvalues, guess=c(0.75*dwi.mat[i,1], 0.05))
-    if(fit$info < 4) {
-      S0[i] <- fit$S0 ; D[i] <- fit$D
+  }
+  for (i in 1:nvoxels) {
+    fit <- adc.lm(dwi.mat[i,], bvalues, guess=c(0.75*dwi.mat[i,1], 0.001))
+    if (fit$info < 4) {
+      S0[i] <- fit$S0
+      D[i] <- fit$D
     } else {
       S0[i] <- D[i] <- NA
     }
   }
 
-  if (verbose)
+  if (verbose) {
     cat("  Reconstructing results...", fill=TRUE)
+  }
   S0.array <- D.array <- array(NA, dim(dwi)[1:3])
   S0.array[dwi.mask] <- S0
   D.array[dwi.mask] <- D
